'===============================================================================================
'Initiative Tracking Tools
'Here's how to use these chat commands.
'
'First, the GM should use /initdice to Set the Type of dice to roll For initiative.  Any valid dice combination can be used.
'example: "/initdice 1d10" tells these functions to use 1d10 For initiative rolls.  You only need to
'do this once.  This information will be saved from session to session as a 'custom field' in the session object.
'The default dice type is d20, so if you dont use this command, a d20 will be used by default.
'
'Then, each player should do /initmod to set their initiative modifier for their character.  For example,
'if a character named Jair is +3 to initiative, the command would be "/initmod Jair, 3". Again, this
'information is saved between sessions, so as long as the character is not removed by the player by
'logging off, it will already be set for the next game
'
'To roll initiatives, the GM can just use /initall.  This will roll & sort the initiatives for each
'character.
'
'To see what the current set of initiatives looks like, just do /init.
'
'===============================================================================================

#commandset Initiative Tools

#register /init Init use "/init <i>character, init_value</i>" to roll init, or "/init" to see the current inits. If <i>init_value</i> is a number, that character's initiative is set to that value.  If it is a dice expression in braces, such as {1d10+2}, the expression is evaluated and init is set to the result.
#register /clearinits ClearInits GM Only. Clears all the current initiative settings
#register /initmod SetInitModifier Sets a character's init modifier.  use "/initmod character, modifier"
#register /initsel InitSelected GM only.  Rolls initiative For any selected miniatures On the server.  Uses any init modifiers that have been Set
#register /initall InitAll GM only.  Rolls init For all characters.  Uses any init modifiers that have been Set using the <i>/initmod</i> chat Command.
#register /initdice SetInitDice GM only.  Sets the dice expression to be used when rolling inits.  Defaults to 1d20.

Sub Init
	cs = GetChatSession()
	ci = cs.GetLastItem()

	ci.Text = Trim( ci.Text)
	If Len( ci.Text) > 0 Then
		RollInit
	Else
		ShowInits
	End If
End Sub


Sub RollInit
	cs = GetChatSession()
	ci = cs.GetLastItem()

	sLine = ci.Text
	ci.Text = sLine & "<i>[Command not understood<br> - use<br><ul><li>/init<li>/init player_name<li>/init player_name<li>dice_expression<li>/init player_name, number</ul>]</i>"

	sChar = ""
	nBonus = 0
	sBonus = ""
	
	
	
	n = InStr( sLine, ",")
	If n > 0 Then
		sChar = Trim( Left( sLine, n-1))
		sBonus = Trim( Mid( sLine, n+1))
		If Len( sBonus) > 0 Then
			nBonus = EvalDice( sBonus)
		End If
		'If IsNumeric( sBonus) Then
		'		nBonus = CInt( sBonus)
		'End If
	Else
		sChar = Trim( sLine)
	End If
	
	oPlayer = GetCurrentPlayer()
	'If oPlayer <> NULL Then
	If NotNull( oPlayer) Then
		oChar = oPlayer.GetCharacter( sChar)
		If NotNull( oChar) Then
			If Len( Trim( sBonus)) < 1 Then
				sBonus = "1d20"
			End If
			nInit = EvalDice( sBonus)
			oChar.SetField "initroll", CStr( nInit)

			If InStr( LCase(sBonus), "d") < 1 Then
				ci.Text = sChar & " sets initiative to <b>" & CStr( nInit) & "</b>"
			Else
				ci.Text = sChar & " rolls <b>" & CStr( nInit) & "</b> for initiative (" & sBonus & ")"
			End If
		Else
			ci.Text = "init " & sLine & "<i>Unknown Character '" & sChar & "'</i>"
		End If
	Else
		ci.Text = "init " & sLine & "<i>No Current Player</i>"
	End If
End Sub


Sub ShowInits
	cs = GetChatSession()
	ci = cs.GetLastItem()
	os = GetCurrentSession()
	
	Dim aInitList(100)
	Dim nInitCount
	
	nInitCount = 0
	
	os.Sync
	n = os.GetSyncCharCount()
	For i = 1 to n
		c = os.GetSyncChar( i)
		sInit = c.GetField( "initroll")
		If Len( Trim( sInit)) > 0 Then
			sInit = String(3 - Len( sInit), "0") & sInit
			aInitList(nInitCount) = sInit & "  " & c.Name
			nInitCount = nInitCount + 1
		End If
	Next
	
	If nInitCount > 0 Then
		QuickSort aInitList, nInitCount+1, false
	End If
	
	
	r = r & "<br><font color='" & os.GM.ChatColor & "'>"

	sInitDice = os.GetField( "initdice")
	If Len( sInitDice) > 0 Then
		r = r & "(init dice set to: " & sInitDice & ")<br>"
	End If


	'r = r & "<br>Current Inits are...<br>"
	For i = 1 to nInitCount
		'trim away leading zeros
		sInit = Left( aInitList(i-1), 3)
		If Left( sInit, 1) = "0" Then
			sInit = " " & Mid( sInit, 2)
		End If

		If Left( sInit, 2) = " 0" Then
			sInit = "  " & Mid( sInit, 3)
		End If
		
		r = r & sInit & Mid( aInitList(i-1), 4) & "<br>"
	Next
	r = r & "</font>"
	
	ci.PlayerFrom = "Current Initiatives are"
	ci.Action = ""
	ci.Text = r
End Sub

Sub ClearInits
	cs = GetChatSession()
	ci = cs.GetLastItem()
	os = GetCurrentSession()
	
	op = GetCurrentPlayer()
	If op.IsGM Then
		os.Sync
		n = os.GetSyncCharCount()
		For i = 1 to n
			c = os.GetSyncChar( i)
			sInit = c.GetField( "initroll")
			If Len( Trim( sInit)) > 0 Then
				c.SetField "initroll", ""
			End If
		Next

		ci.Action = "reports"
		ci.Text = "initiatives cleared"
	Else
		ci.PlayerTo = ci.PlayerFrom
		ci.Text = "<i>[Sorry, only the GM can clear the initiatives!]</i>"
	End If
	
End Sub



Sub SetInitModifier
	cs = GetChatSession()
	ci = cs.GetLastItem()

	sChar = ""
	sMod = ""
	bSet = False
	
	
	'Get command line
	aLine = Split( ci.Text, ",")
	If UBound( aLine) > -1 Then
		sChar = Trim( aLine(0))
	End If
	If UBound( aLine) > 0 Then
		sMod = Trim( aLine(1))
	End If


	oPlayer = GetCurrentPlayer()
	If NotNull( oPlayer) Then
		oChar = oPlayer.GetCharacter( sChar)
		If NotNull( oChar) Then
			oChar.SetField "initmod", sMod
			bSet = true
		End If
	End If
	
	If bSet Then
		ci.Text = sChar & " sets init modifier to " & sMod
	Else
		ci.Text = ci.Text & "<i>[Command not understood, or you're trying to set the initiative modifier for a character that is not yours]</i> " & sChar & sMod
	End If


End Sub

Sub InitSelected
	RollInitsSelected( False)
End Sub

Sub InitAll
	RollInitsSelected( True)
End Sub

Sub RollInitsSelected( bAll)
	cs = GetChatSession()
	ci = cs.GetLastItem()
	os = GetCurrentSession()
	sLine = ""
	sInitDice = ""


	sInitDice = os.GetField( "initdice")
	If Len( sInitDice) < 1 Then
		sInitDice = "1d20"
	End If 
	
	op = GetCurrentPlayer()
	If op.IsGM Then
		os.Sync
		n = os.GetSyncCharCount()
		For i = 1 to n
			c = os.GetSyncChar( i)
			If (bAll = true) or (c.Selected = true) Then
				sMod = c.GetField( "initmod")
				nMod = 0
				If IsNumeric( sMod) Then
					nMod = CInt( sMod)
				End If
				nInit = EvalDice( sInitDice)
				nInit = nInit + nMod
				c.SetField "initroll", CStr( nInit)
				'sLine = sLine & CStr( nInit) & " " & c.Name & " (" & CStr( nMod) & " modifier)<br>"
			End If
		Next
		ShowInits
		'ci.Action = "reports"
		'ci.Text = "<br>Initiative(s) rolled<br>" & sLine
	Else
		ci.PlayerTo = ci.PlayerFrom
		ci.Text = "<i>[Sorry, only the GM can do that!]</i>"
	End If
	
End Sub


Sub SetInitDice
	cs = GetChatSession()
	ci = cs.GetLastItem()
	os = GetCurrentSession()
	sLine = ""
	
	op = GetCurrentPlayer()
	If op.IsGM Then
		os.SetField "initdice", ci.Text
		ci.Action = "sets initiative dice type to " + ci.Text
	Else
		ci.PlayerTo = ci.PlayerFrom
		ci.Text = "<i>[Sorry, only the GM can do that!]</i>"
	End If
	
End Sub

'==== Sorting Routines found on Usenet =====
'Just call QuickSort with Array and its size 
'and True for Ascending False for descending 
Function QuickSort(strAr, nSize, bAscending)
 rc = TRUE
 If (nSize > 1) Then
  low = 0
  high = nSize
  'Call the QuickSort Recursive Function
  QuickSortRecursive strAr,low,high,bAscending
 Else
  rc = FALSE
 End If
 QuickSort =  rc
End Function

'==================== QUICK SORT RECURSIVE 
Function QuickSortRecursive(strAr,  d, h, bAscending)
 i = h
 j = d

 str = strAr(((d+h) / 2))
 do
  If (bAscending) Then
   while (strAr(j) < str)
    j = j + 1
   wend
   while (strAr(i) > str)
    i = i - 1
   wend
  Else
   while (strAr(j) > str) j = j + 1
   wend
   while (strAr(i) < str) i = i - 1
   wend
  End If

  If ( i >= j ) Then
   If ( i <> j ) Then
    zal = strAr(i)
    strAr(i) = strAr(j)
    strAr(j) = zal
   End If

   i = i - 1
   j = j + 1
  End If
 loop while (j <= i)

 If (d < i) Then
  QuickSortRecursive strAr,d,i,bAscending
 End If
 If (j < h) Then
  QuickSortRecursive strAr,j,h,bAscending
 End If
End Function



